<?php
// require config.php!
class ContentHandler {
	private $_defaultPath;
	private $_filePath;
	private $_templateHtml;
	private $_title;
	private $_content;
	private $_newTemplateHtml;
	private $_siteTitleFile;
	private $_siteTitle;
	private $_backupManager;
	private $_content_block_pattern;
	private $_content_marker_start;
	private $_content_marker_end;
	
	function __construct($str_path) {
		global $ne2_config_info;
		
		$this->_siteTitleFile = $ne2_config_info['app_path'] . $ne2_config_info['current_site_title_file'];
		$this->_defaultPath = $ne2_config_info['app_path'] . 'data/templates/seitenvorlage.html';
		if(!file_exists($this->_defaultPath)) {
			$this->_defaultPath = $ne2_config_info['app_path'] . 'data/templates/_seitenvorlage.html'; // fail-safe
		}
		$this->_newTemplateHtml = '';
		$this->_content_block_pattern = '/(<a name="contentmarke" id="contentmarke"><\/a>)((\r|\n|\r\n|.)*?)(<hr id="vorfooter" \/>)/i';
		$this->_content_marker_start = '<a name="contentmarke" id="contentmarke"></a>';
		$this->_content_marker_end = '<hr id="vorfooter" />';
		
		$this->_filePath = $str_path;
		if(file_exists($this->_filePath)) {
//			if(($this->_templateHtml = file_get_contents($this->_filePath)) == FALSE) {
			if(($this->_templateHtml = $this->file_get_contents_utf8($this->_filePath)) == FALSE) {
				throw new Exception('Unable to open file: ' . $this->_filePath);
			}
		} else { // try default path
			if(($this->_templateHtml = file_get_contents($this->_defaultPath)) == FALSE) {
				throw new Exception('Unable to open file: ' . $this->_filePath);
			}
		}
		
		if(file_exists($this->_siteTitleFile)) {
			if(($this->_siteTitle = trim(file_get_contents($this->_siteTitleFile))) == FALSE) {
//				throw new Exception('Unable to open Site-Title File!');
				$this->_siteTitle = '';
			} else {
				$this->_siteTitle .= ': ';
			}
		}
	}
	
	private function file_get_contents_utf8($fn) {
		$content = file_get_contents($fn);
		$content = str_ireplace('<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />', '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />', $content);
		return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', TRUE));
	}
	
	public function setBackupManager($backupManager) {
		$this->_backupManager = $backupManager;
	}
	
	public function setContents($newTitle, $newContent) {
		$this->_title = htmlentities($newTitle, ENT_COMPAT, 'UTF-8');
		$this->_content = $newContent;
		if($this->_content == '') {
			$str_pattern = $this->_content_block_pattern;
			preg_match($str_pattern, $this->_templateHtml, $matches);
			$this->_content = $matches[2];
		}
	}
	
	private function replaceTitles() {
		// <title>...</title>
		$pattern1 = '/<title>((.)*?)<\/title>/i';
		$this->_templateHtml = preg_replace($pattern1, '<title>' . $this->_siteTitle . $this->_title . '</title>', $this->_templateHtml);
		
		// <h1>...</h1>
		$pattern2 = '/(<div id="titel">)((\n|.)*?)(<\/div>)/i';
		$this->_templateHtml = preg_replace($pattern2, '$1<h1>' . $this->_title . '</h1>$4', $this->_templateHtml);
	}
	
	private function replaceContent() {
		$str_pattern = $this->_content_block_pattern;
		$this->_templateHtml = preg_replace($str_pattern, '$1' . "\n" . $this->_content . "\n" . '$4', $this->_templateHtml);
	}
	
	private function _replaceContentEx() {
		$start_pos = mb_strpos($this->_templateHtml, $this->_content_marker_start);
		$end_pos = mb_strpos($this->_templateHtml, $this->_content_marker_end);
		if($start_pos === FALSE || $end_pos === FALSE) {
			return;
		}
		$start_pos += mb_strlen($this->_content_marker_start);
		$part_head = mb_substr($this->_templateHtml, 0, $start_pos);
		$part_tail = mb_substr($this->_templateHtml, $end_pos);
		$this->_templateHtml = $part_head . $this->_content . $part_tail;
	}
	
	// Do the replacement
	public function ReplaceContents($backup = TRUE) {
//		$this->replaceTitles();
//		$this->_replaceContentEx();
		
		if($backup) {
			$this->_backupManager->backup($this->_filePath);
		}
		$this->SaveAsDraft();
//		file_put_contents($this->_filePath, $this->_templateHtml);
		$draft_path = $this->_filePath . '.buffer';
		if(file_exists($draft_path)) {
			copy($draft_path, $this->_filePath);
			@unlink($draft_path);
		}
	}
	
	public function SaveAsDraft() {
		$draft_path = $this->_filePath . '.buffer';
		if(file_exists($draft_path)) {
			$this->_templateHtml = $this->file_get_contents_utf8($draft_path);
		}
		
		$this->replaceTitles();
		$this->_replaceContentEx();
		
		file_put_contents($draft_path, $this->_templateHtml);
	}
	
	public function GetContentHtml() {
		$pattern = $this->_content_block_pattern;
		preg_match($pattern, $this->_templateHtml, $matches);
		// error
		if(count($matches) < 3) {
			return '<span style="color:red;">INFO: No content tags were found (' . count($matches) . '), this page is possibly not produced from templates.</span>';
		}
		$ret = $matches[2];
		return $ret;
	}
	
	public function getContentHtmlEx() {
		$start_pos = mb_strpos($this->_templateHtml, $this->_content_marker_start);
		$end_pos = mb_strpos($this->_templateHtml, $this->_content_marker_end);
		if($start_pos === FALSE || $end_pos === FALSE) {
			return '<span style="color:red;">INFO: No content tags were found, this page is possibly not produced from templates.</span>';
		}
		$len = mb_strlen($this->_templateHtml);
		$start_pos += mb_strlen($this->_content_marker_start);
		$real_content = mb_substr($this->_templateHtml, $start_pos, $end_pos - $start_pos);
		return $real_content;
	}
	
	public function change_ssi_head($new_head_filename) {
		// get content in <head>
		$start_pos = mb_strpos($this->_templateHtml, '<head>');
		$end_pos = mb_strpos($this->_templateHtml, '</head>');
		$start_pos += mb_strlen('<head>');
		$head_content = trim(mb_substr($this->_templateHtml, $start_pos, $end_pos - $start_pos));
		$head_content_array = explode("\n", $head_content);
		$new_head_content = '';
		foreach($head_content_array as $line) {
			if(mb_strpos($line, '<!--#include virtual') !== FALSE) {
				$new_head_content .= '<!--#include virtual="/ssi/' . $new_head_filename . '" -->' . "\n";
			} else {
				$new_head_content .= $line . "\n";
			}
		}
		
		$part_head = mb_substr($this->_templateHtml, 0, $start_pos);
		$part_tail = mb_substr($this->_templateHtml, $end_pos);
		$this->_templateHtml = $part_head . "\n" . $new_head_content . $part_tail;
		
		$draft_path = $this->_filePath . '.buffer';
		file_put_contents($draft_path, $this->_templateHtml);
	}
	
	private function pcre_error_deocde() {
		$r = '';
		switch(preg_last_error()) {
		case PREG_NO_ERROR:
			$r = "pcre_error: PREG_NO_ERROR!\n";
			break;
		case PREG_INTERNAL_ERROR:
			$r = "pcre_error: PREG_INTERNAL_ERROR!\n";
			break;
		case PREG_BACKTRACK_LIMIT_ERROR:
			$r = "pcre_error: PREG_BACKTRACK_LIMIT_ERROR!\n";
			break;
		case PREG_RECURSION_LIMIT_ERROR:
			$r = "pcre_error: PREG_RECURSION_LIMIT_ERROR!\n";
			break;
		case PREG_BAD_UTF8_ERROR:
			$r = "pcre_error: PREG_BAD_UTF8_ERROR!\n";
			break;
		case PREG_BAD_UTF8_OFFSET_ERROR:
			$r = "pcre_error: PREG_BAD_UTF8_OFFSET_ERROR!\n";
			break;
		}
		return $r;
	}
}
?>
