<?php
class FileManager {
	
	public function __construct() {
	}
	
	private function formatBytes($bytes, $precision = 2) {
		$units = array('B', 'KB', 'MB', 'GB', 'TB');
		
		$bytes = max($bytes, 0);
		$pow = floor(($bytes ? log($bytes) : 0) / log(1024));
		$pow = min($pow, count($units) - 1);
		
		$bytes /= pow(1024, $pow);
		
		return round($bytes, $precision) . ' ' . $units[$pow];
	}
	
	public function createSubFolder($ppath, $fname) {
		$compl_name = $ppath . '/' . $fname;
		return mkdir($compl_name);
	}
	
	public function loadFolderTree($path, $is_root = TRUE) {
		$da = array(
			'dir_name' => basename($path),
			'dir_path' => $path,
			'sub' => array()
		);
		if($is_root) {
			$dl = array_diff(scandir($path), array('.', '..', 'js', 'Smarty', 'univis', 'vkapp')); // here insert excluded folders
		} else {
			$dl = array_diff(scandir($path), array('.', '..', 'NavEditor2')); // here insert excluded folders
		}
		foreach($dl as $di) {
			if(is_dir($path . '/' . $di)) {
				array_push($da['sub'], $this->loadFolderTree($path . '/' . $di, FALSE));
			}
		}
		return $da;
	}
	
	private function build_file_icon($file_name) {
		$fa = explode('.', $file_name);
		$ext = $fa[count($fa) - 1];
		$icon_path = $_SERVER['DOCUMENT_ROOT'] . '/img/links/';
		if(file_exists($icon_path . $ext . '.gif')) {
			return "<img alt='$ext' src='/img/links/$ext.gif' border='0' width='16' height='16' />";
		} else {
			return "<img alt='' src='/img/links/extern.gif' border='0' width='16' height='16' />";
		}
	}
	
	public function getFileList($path) {
		$fl = array();
		
		$dl = array_diff(scandir($path), array('.', '..'));
		foreach($dl as $di) {
			$di = $path . '/' . $di;
			if((is_file($di)) && (substr(basename($di), 0, 6) != 'thumb_')) {
				$f = array(
					'icon' => '',
					'file_name' => '',
					'extension' => '',
					'file_size' => '',
					'access_time' => ''
				);
				$ap = pathinfo($di);
				$f['icon'] = $this->build_file_icon($ap['basename']);
				$f['file_name'] = $ap['basename'];
				$f['extension'] = $ap['extension'];
				$f['file_size'] = $this->formatBytes(filesize($di));
				$f['access_time'] = date("d.m.Y H:i:s", filemtime($di));
				array_push($fl, $f);
			}
		}
		return $fl;
	}
	
	public function getFileInfo($fpath) {
		$editable_exts = array('txt', 'html', 'shtml', 'css', 'js', 'pl', 'conf', 'php');
		$r = array(
			'file_name' => '',
			'file_size' => '',
			'modified_time' => '',
			'thumb_name' => '',
			'url' => '',
			'editable' => FALSE
		);
		if(file_exists($fpath)) {
			$pi = pathinfo($fpath);
			$r['file_name'] = $pi['basename'];
			$r['file_size'] = filesize($fpath);
			$thumb_path = substr($fpath, 0, strrpos($fpath, '/')) . '/thumb_' . $pi['basename'];
			if(file_exists($thumb_path)) {
				$r['thumb_name'] = str_replace($_SERVER['DOCUMENT_ROOT'], '', $thumb_path);
			}
			$r['url'] = str_replace($_SERVER['DOCUMENT_ROOT'], '', $fpath);
			$ext = $pi['extension'];
			if(in_array(strtolower($ext), $editable_exts)) {
				$r['editable'] = TRUE;
			}
		}
		return $r;
	}
	
	public function deleteFile($file_path) {
		// if thumb image exists, also delete
		$file_name_only = basename($file_path);
		$thumb_name = 'thumb_' . $file_name_only;
		$thumb_path = str_replace($file_name_only, $thumb_name, $file_path);
		if(file_exists($thumb_path)) {
			@unlink($thumb_path);
		}
		@unlink($file_path);
	}
	
	public function getFileContent($file_path) {
		if(file_exists($file_path)) {
			$c = file_get_contents($file_path);
			// replace ssi tags
			$c = str_replace(array('<!--#', '<!--', '-->'), array('<comment_ssi>', '<comment>', '</comment>'), $c);
			return $c;
		} else {
			return $file_path;
		}
	}
	
	public function setFileContent($file_path, $content) {
		if(file_exists($file_path)) {
			$content = stripslashes($content);
			$content = str_replace(array('<comment_ssi>', '<comment>', '</comment>'), array('<!-' . '-#', '<!--', '-->'), $content);
			file_put_contents($file_path, $content);
		}
	}
		
}
?>